#!/bin/sh

# Remove all forwardings (LAN -> WAN should be the only one by default)
uci show firewall | grep -o "firewall\.\@forwarding\[[0-9]\+\]" | uniq | xargs uci delete
uci commit

# Re-add forwardings between ygg and lan
uci add firewall forwarding
uci set firewall.@forwarding[-1].dest='yggdrasil'
uci set firewall.@forwarding[-1].src='lan'
uci commit

# Enable masquerading on ygg interface
uci set firewall.yggdrasil.masq='1'
# Make sure that autoygg traffic is added to the yggdrasil 'zone'. Needed to
# allow traffic to the autoygg server from machines behind the meshnode.
uci show firewall.yggdrasil.network |grep -q autoygg
if [ $? == 1 ]; then
  uci add_list firewall.yggdrasil.network='autoygg'
fi
uci commit

# Allow Iperf3 testing TCP/UDP on port 5201
uci add firewall rule
uci set firewall.@rule[-1].name='Iperf3 Throughput / Bandwidth'
uci set firewall.@rule[-1].src='*'
uci set firewall.@rule[-1].dest_port='5201'
uci set firewall.@rule[-1].target='ACCEPT'
uci commit

# Allow a default-disabled firewall rule for prometheus metrics access from
# prometheus.massmesh.net
uci add firewall rule
uci set firewall.@rule[-1].name='Metrics access from prometheus.massmesh.net'
uci set firewall.@rule[-1].src='yggdrasil'
uci add_list firewall.@rule[-1].proto='tcp'
uci set firewall.@rule[-1].family='ipv6'
uci set firewall.@rule[-1].dest_port='9100-9101'
uci add_list firewall.@rule[-1].src_ip='201:506e:60d6:bd66:e35c:606:4883:ea9a'
uci set firewall.@rule[-1].target='ACCEPT'
uci set firewall.@rule[-1].enabled=0
uci commit

# Add yggdrasil peers
peers="
# USA
tcp://gateway1.clearnet.massmesh.net:1617
tcp://50.236.201.218:56088
tcp://[2607:f1c0:1801:d4::1]:61216
tcp://lancis.iscute.moe:49273
tls://lancis.iscute.moe:49274
tcp://longseason.1200bps.xyz:13121
tls://longseason.1200bps.xyz:13122
tcp://213.188.210.9:10010
tls://213.188.210.9:10020
tcp://213.188.199.150:10010
tls://213.188.199.150:10020

# BRA
tcp://45.231.133.188:58301

# Canada
tcp://195.123.245.146:7743
tcp://37.205.14.171:46370
tls://217.195.164.4:10531

# Finland
tls://65.21.57.122:61995
tls://fi1.servers.devices.cwinfo.net:61995
tls://[2a01:4f9:c010:664d::1]:61995

# Netherlands
tcp://ygg-nl.incognet.io:8883
tls://ygg-nl.incognet.io:8884
tls://[2a00:7b80:3013:bc5::]:62486

# France
tcp://s2.i2pd.xyz:39565
tls://s2.i2pd.xyz:39575
tcp://62.210.85.80:39565
tls://62.210.85.80:39575
tcp://[2001:470:1f13:e56::64]:39565
tls://[2001:470:1f13:e56::64]:39575
"

peers=$(echo -e "$peers" | grep -E "^(tcp|tls)")

for peer in ${peers//\\n/ }
do
  uci add yggdrasil interface_peer
  uci set yggdrasil.@interface_peer[-1].uri="$peer"
  uci set yggdrasil.@interface_peer[-1].interface=`uci get network.wan.device`
done
uci commit

# Add yggdrasil tunnel routing config
uci set yggdrasil.yggdrasil.TunnelRouting_Enable=1
uci add yggdrasil ipv4_local_subnet
uci set yggdrasil.@ipv4_local_subnet[-1].subnet='0.0.0.0/0'
uci commit

# Set ula
uci set network.globals.ula_prefix=$(ygguci get | yggdrasil -useconf -normaliseconf -json | yggdrasil -useconf -subnet)
uci commit
